'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var constants = require('../common/constants.js');
var change = require('./internal/change.js');
var message = require('./internal/message.js');
var room = require('./internal/room.js');
var serializer = require('./internal/serializer.js');
var websocket = require('./websocket.js');

class WebsocketClient {
  #sid = 0;
  #socket$;
  #joinRequest$ = new rxjs.BehaviorSubject(null);
  #joinResult$ = new rxjs.BehaviorSubject(null);
  #close$ = new rxjs.Subject();

  constructor({
    token = '',
    debug
  }) {
    this.#socket$ = new rxjs.Observable(subscriber => {
      const socket = new websocket.RxWebSocket(constants.constants.websocket.endpoint, {
        clientOptions: {
          headers: {
            Origin: constants.constants.websocket.origin,
            Cookie: `connect.sid=${token}`
          }
        },
        serializer: serializer.socketIoMessageSerializer,
        deserializer: serializer.socketIoMessageDeserializer,
        debug
      });
      const connected$ = socket.messageOf(message.isInitializedMessage);
      connected$.subscribe(({
        data
      }) => {
        // send ack
        socket.send(message.packetTypes.connectedMessage).messageOf(message.isConnectedMessage).subscribe(() => {
          // set ping
          rxjs.interval(data.pingInterval).pipe(operators.takeUntil(socket)).subscribe(() => socket.send(message.packetTypes.ping));
          subscriber.next(socket);
        });
      }); // TODO: use share with { resetOnComplete: true } to allow re-connect

      socket.pipe(operators.ignoreElements(), operators.takeUntil(this.#close$)).subscribe(subscriber);
      return () => socket.close();
    }).pipe(operators.shareReplay(1), operators.take(1));
    this.#joinRequest$.pipe(operators.takeUntil(this.#close$), operators.concatMap(room$1 => rxjs.iif(() => room.isSameRoom(this.#joinResult$.getValue(), room$1), rxjs.of(room$1), rxjs.of(room$1).pipe(operators.delayWhen(() => this.#send({
      method: 'room:join',
      // TODO(feat): can join without pageId
      data: {
        pageId: room$1 === null || room$1 === void 0 ? void 0 : room$1.pageId,
        projectId: room$1 === null || room$1 === void 0 ? void 0 : room$1.projectId,
        projectUpdatesStream: false
      }
    })))))).subscribe(this.#joinResult$);
  }

  async commit({
    userId,
    projectId,
    pageId,
    parentId,
    changes
  }) {
    return rxjs.lastValueFrom(rxjs.concat(this.#joinResult$.pipe(operators.tap(() => this.#joinRequest$.next({
      projectId,
      pageId
    })), operators.skip(1), // ignore cache of BehaviorSubject
    operators.take(1)), this.#send({
      method: 'commit',
      data: {
        kind: 'page',
        userId,
        projectId,
        pageId,
        parentId,
        changes: change.createChanges(changes, userId),
        cursor: null,
        freeze: true
      }
    })));
  }

  close() {
    this.#close$.next(undefined);
  } // TODO(feat): add error handling


  #send(data) {
    const sid = this.#sid++;
    return this.#socket$.pipe(operators.tap(socket => socket.send([sid, data])), operators.mergeMap(socket => socket.messageOf(message.isResponseMessageOf(sid), constants.constants.websocket.responseTimeout)));
  }

}

exports.WebsocketClient = WebsocketClient;
//# sourceMappingURL=websocket-client.js.map
