'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var utils = require('../common/utils.js');
var debug = require('./internal/debug.js');

const defaultSerializer = data => String(data);

const defaultDeserializer = message => message.data;

class RxWebSocket extends rxjs.Subject {
  #websocket;
  #message$;
  #serializer;

  constructor(url, {
    protocols,
    clientOptions,
    serializer = defaultSerializer,
    deserializer = defaultDeserializer,
    debug: debug$1 = false
  }) {
    super();
    const websocket = utils.isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);

    if (debug$1) {
      debug.debugWebsocket(websocket);
    }

    const open$ = rxjs.fromEvent(websocket, 'open').pipe(operators.take(1));
    const error$ = rxjs.fromEvent(websocket, 'error').pipe(operators.take(1));
    const close$ = rxjs.fromEvent(websocket, 'close').pipe(operators.take(1));
    const message$ = rxjs.fromEvent(websocket, 'message').pipe(operators.takeUntil(close$), operators.shareReplay(1));
    const events$ = rxjs.merge(open$, message$, error$.pipe(operators.mergeMap(rxjs.throwError)), close$).pipe(operators.takeUntil(close$), operators.shareReplay(1));
    this.#websocket = websocket;
    this.#message$ = message$.pipe(operators.map(deserializer));
    this.#serializer = serializer;
    events$.subscribe(this);
  }

  send(data) {
    this.#websocket.send(this.#serializer(data));
    return this;
  }

  get message() {
    return this.#message$;
  }

  messageOf(predicate, timeoutMs) {
    const response = this.#message$.pipe(operators.first(predicate));
    return timeoutMs ? response.pipe(operators.timeout(timeoutMs)) : response;
  }

  close() {
    this.#websocket.close();
    this.complete();
  }

}

exports.RxWebSocket = RxWebSocket;
//# sourceMappingURL=websocket.js.map
