import { BehaviorSubject, Subject, Observable, interval, iif, of, lastValueFrom, concat } from 'rxjs';
import { takeUntil, ignoreElements, shareReplay, take, concatMap, delayWhen, tap, skip, mergeMap } from 'rxjs/operators';
import { constants } from '../common/constants.js';
import { createChanges } from './internal/change.js';
import { isInitializedMessage, packetTypes, isConnectedMessage, isResponseMessageOf } from './internal/message.js';
import { isSameRoom } from './internal/room.js';
import { socketIoMessageSerializer, socketIoMessageDeserializer } from './internal/serializer.js';
import { RxWebSocket } from './websocket.js';

class WebsocketClient {
    #sid = 0;
    #socket$;
    #joinRequest$ = new BehaviorSubject(null);
    #joinResult$ = new BehaviorSubject(null);
    #close$ = new Subject();
    constructor({ token = '', debug, }) {
        this.#socket$ = new Observable((subscriber) => {
            const socket = new RxWebSocket(constants.websocket.endpoint, {
                clientOptions: { headers: { Origin: constants.websocket.origin, Cookie: `connect.sid=${token}` } },
                serializer: socketIoMessageSerializer,
                deserializer: socketIoMessageDeserializer,
                debug,
            });
            const connected$ = socket.messageOf(isInitializedMessage);
            connected$.subscribe(({ data }) => {
                // send ack
                socket
                    .send(packetTypes.connectedMessage)
                    .messageOf(isConnectedMessage)
                    .subscribe(() => {
                    // set ping
                    interval(data.pingInterval)
                        .pipe(takeUntil(socket))
                        .subscribe(() => socket.send(packetTypes.ping));
                    subscriber.next(socket);
                });
            });
            // TODO: use share with { resetOnComplete: true } to allow re-connect
            socket.pipe(ignoreElements(), takeUntil(this.#close$)).subscribe(subscriber);
            return () => socket.close();
        }).pipe(shareReplay(1), take(1));
        this.#joinRequest$
            .pipe(takeUntil(this.#close$), concatMap((room) => iif(() => isSameRoom(this.#joinResult$.getValue(), room), of(room), of(room).pipe(delayWhen(() => this.#send({
            method: 'room:join',
            // TODO(feat): can join without pageId
            data: { pageId: room?.pageId, projectId: room?.projectId, projectUpdatesStream: false },
        }))))))
            .subscribe(this.#joinResult$);
    }
    async commit({ userId, projectId, pageId, parentId, changes, }) {
        return lastValueFrom(concat(this.#joinResult$.pipe(tap(() => this.#joinRequest$.next({ projectId, pageId })), skip(1), // ignore cache of BehaviorSubject
        take(1)), this.#send({
            method: 'commit',
            data: { kind: 'page', userId, projectId, pageId, parentId, changes: createChanges(changes, userId), cursor: null, freeze: true },
        })));
    }
    close() {
        this.#close$.next(undefined);
    }
    // TODO(feat): add error handling
    #send(data) {
        const sid = this.#sid++;
        return this.#socket$.pipe(tap((socket) => socket.send([sid, data])), mergeMap((socket) => socket.messageOf(isResponseMessageOf(sid), constants.websocket.responseTimeout)));
    }
}

export { WebsocketClient };
//# sourceMappingURL=websocket-client.js.map
