import { Subject, fromEvent, merge, throwError } from 'rxjs';
import { take, takeUntil, shareReplay, mergeMap, map, first, timeout } from 'rxjs/operators';
import { isBrowser } from '../common/utils.js';
import { debugWebsocket } from './internal/debug.js';

const defaultSerializer = (data) => String(data);
const defaultDeserializer = (message) => message.data;
class RxWebSocket extends Subject {
    #websocket;
    #message$;
    #serializer;
    constructor(url, { protocols, clientOptions, serializer = defaultSerializer, deserializer = defaultDeserializer, debug = false, }) {
        super();
        const websocket = isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);
        if (debug) {
            debugWebsocket(websocket);
        }
        const open$ = fromEvent(websocket, 'open').pipe(take(1));
        const error$ = fromEvent(websocket, 'error').pipe(take(1));
        const close$ = fromEvent(websocket, 'close').pipe(take(1));
        const message$ = fromEvent(websocket, 'message').pipe(takeUntil(close$), shareReplay(1));
        const events$ = merge(open$, message$, error$.pipe(mergeMap(throwError)), close$).pipe(takeUntil(close$), shareReplay(1));
        this.#websocket = websocket;
        this.#message$ = message$.pipe(map(deserializer));
        this.#serializer = serializer;
        events$.subscribe(this);
    }
    send(data) {
        this.#websocket.send(this.#serializer(data));
        return this;
    }
    get message() {
        return this.#message$;
    }
    messageOf(predicate, timeoutMs) {
        const response = this.#message$.pipe(first(predicate));
        return timeoutMs ? response.pipe(timeout(timeoutMs)) : response;
    }
    close() {
        this.#websocket.close();
        this.complete();
    }
}

export { RxWebSocket };
//# sourceMappingURL=websocket.js.map
